@@ -54,8 +54,6 @@ class DummyOptions:
         self.privsdropped = None
         self.logs_reopened = False
         self.environment_processed = False
-        self.select_result = [], [], []
-        self.select_error = None
         self.write_accept = None
         self.write_error = None
         self.tempfile_name = '/foo/bar'
@@ -65,10 +63,12 @@ class DummyOptions:
         self.openreturn = None
         self.readfd_result = ''
         self.parse_warnings = []
+        self.parse_infos = []
         self.serverurl = 'http://localhost:9001'
         self.changed_directory = False
         self.chdir_error = None
         self.umaskset = None
+        self.poller = DummyPoller(self)
 
     def getLogger(self, *args, **kw):
         logger = DummyLogger()
@@ -142,7 +142,7 @@ class DummyOptions:
     def get_pid(self):
         import os
         return os.getpid()
-        
+
     def check_execv_args(self, filename, argv, st):
         if filename == '/bad/filename':
             from supervisor.options import NotFound
@@ -220,12 +220,6 @@ class DummyOptions:
     def mktempfile(self, prefix, suffix, dir):
         return self.tempfile_name
 
-    def select(self, r, w, x, timeout):
-        import select
-        if self.select_error:
-            raise select.error(self.select_error)
-        return self.select_result
-
     def remove(self, path):
         import os
         if self.remove_error:
@@ -267,7 +261,7 @@ class DummyLogger:
         if kw:
             msg = msg % kw
         self.data.append(msg)
-        
+
     def reopen(self):
         self.reopened = True
     def close(self):
@@ -351,7 +345,7 @@ class DummySocketManager:
 
     def get_socket(self):
         return DummySocket(self._config.fd)
-        
+
 class DummyProcess:
     # Initial state; overridden by instance variables
     pid = 0 # Subprocess pid; 0 when not running
@@ -557,7 +551,7 @@ def makeExecutable(file, substitutions=None):
     import os
     import sys
     import tempfile
-    
+
     if substitutions is None:
         substitutions = {}
     data = open(file).read()
@@ -566,12 +560,12 @@ def makeExecutable(file, substitutions=None):
     substitutions['PYTHON'] = sys.executable
     for key in substitutions.keys():
         data = data.replace('<<%s>>' % key.upper(), substitutions[key])
-    
+
     tmpnam = tempfile.mktemp(prefix=last)
     f = open(tmpnam, 'w')
     f.write(data)
     f.close()
-    os.chmod(tmpnam, 0755)
+    os.chmod(tmpnam, int('755', 8))
     return tmpnam
 
 def makeSpew(unkillable=False):
@@ -605,7 +599,7 @@ class DummyMedusaChannel:
     def set_terminator(self, terminator):
         pass
 
-class DummyRequest(dict):
+class DummyRequest(object):
     command = 'GET'
     _error = None
     _done = False
@@ -634,6 +628,12 @@ class DummyRequest(dict):
     def __setitem__(self, header, value):
         self.headers[header] = value
 
+    def __getitem__(self, header):
+        return self.headers[header]
+
+    def __delitem__(self, header):
+        del self.headers[header]
+
     def has_key(self, header):
         return self.headers.has_key(header)
 
@@ -651,7 +651,7 @@ class DummyRequest(dict):
 
     def get_server_url(self):
         return 'http://example.com'
-        
+
 
 class DummyRPCInterfaceFactory:
     def __init__(self, supervisord, **config):
@@ -826,9 +826,9 @@ class DummySupervisorRPCNamespace:
             raise Fault(xmlrpc.Faults.NOT_RUNNING, 'NOT_RUNNING')
         if name == 'FAILED':
             raise Fault(xmlrpc.Faults.FAILED, 'FAILED')
-        
+
         return True
-    
+
     def stopAllProcesses(self):
         from supervisor import xmlrpc
         return [
@@ -919,6 +919,9 @@ class DummySupervisorRPCNamespace:
     def raiseError(self):
         raise ValueError('error')
 
+    def getXmlRpcUnmarshallable(self):
+        return {'stdout_logfile': None}  # None is unmarshallable
+
     def getSupervisorVersion(self):
         return '3000'
 
@@ -1017,15 +1020,15 @@ class DummyProcessGroup:
 
     def stop_all(self):
         self.all_stopped = True
-        
+
     def get_unstopped_processes(self):
         return self.unstopped_processes
 
     def get_dispatchers(self):
         return self.dispatchers
-        
+
 class DummyFCGIProcessGroup(DummyProcessGroup):
-    
+
     def __init__(self, config):
         DummyProcessGroup.__init__(self, config)
         self.socket_manager = DummySocketManager(config.socket_config)
@@ -1096,7 +1099,7 @@ class DummyDispatcher:
         if self.flush_error:
             raise OSError(self.flush_error)
         self.flushed = True
-                
+
 class DummyStream:
     def __init__(self, error=None):
         self.error = error
@@ -1117,7 +1120,7 @@ class DummyStream:
         pass
     def tell(self):
         return len(self.written)
-        
+
 class DummyEvent:
     def __init__(self, serial='abc'):
         if serial is not None:
@@ -1126,6 +1129,23 @@ class DummyEvent:
     def __str__(self):
         return 'dummy event'
 
+class DummyPoller:
+    def __init__(self, options):
+        self.result = [], []
+        self.closed = False
+
+    def register_readable(self, fd):
+        pass
+
+    def register_writable(self, fd):
+        pass
+
+    def poll(self, timeout):
+        return self.result
+
+    def close(self):
+        self.closed = True
+
 def dummy_handler(event, result):
     pass
 
